class IgConsentUi {
    constructor() {
        this.manager = window.IgConsentManager;
        this.consentHideStorageName = this.manager.config.baseCookieName + 'hide';
        this.elements = new Map();
        this.setElements();
        this.setButtons();
        this.isDisplayed();
    }

    getTemplate() {
const Template = ({
    purpose,
    parent,
    translate
}) => `
    <div class="${!parent ? 'purpose' : ''}">
       <div id="service-${purpose.name}" class="service-${purpose.name} cookieCategory checkbox-container">
             <label for="service-check-${purpose.name}">
                <input type="checkbox" id="service-check-${purpose.name}" name="cookieCategories" value="${purpose.name}" ${parent ? 'data-parent="'+parent+'"': ''} ${purpose.required ? 'disabled="1"' : ''} ${purpose.required ? 'checked="1"' : ''} />
                <div class="checkbox-slider"${purpose.required ? ' data-disabled="1"' : ''}></div>
                <div class="checkbox-content">
                    <span class="cm-list-title">${purpose.title}</span>
		      ${purpose.required ? '<span class="cm-required">(' + translate['alwaysRequired']+ ')</span>' : ''}
                    <p class="cm-list-description">${purpose.description}</p>
                </div>
             </label>
        </div>
     </div>`;
	let html = '';
	for(const purpose of this.manager.purposes) {
	    html += '<div class="purpose-group">';
	    html += Template({purpose: purpose, parent: '', translate: this.manager.translations});
	    if (!purpose.hideServices) {
		for(const service of this.manager.services) {
		    if (service.purpose == purpose.name && (!service.contextualConsentOnly || this.manager.getConsent(service.name)))   {
			html += '<div class="service">' + Template({purpose: service, parent: service.purpose, translate: this.manager.translations}) + '</div>';
		    }
		}
	    }
	    html += '</div>';
	}
	return html;
    }
    isDisplayed() {
        if (!localStorage.getItem(this.consentHideStorageName)) {
            this.elements.get('html').classList.add('ig-consent-show');
        }
    }
    show() {
        this.elements.get('html').classList.add('ig-consent-show');
    }
    showSettings() {
        this.elements.get('html').classList.add('ig-consent-show');
        this.settings();
    }
    hide() {
        localStorage.setItem(this.consentHideStorageName, true);
        this.elements.get('html').classList.remove('ig-consent-show');
    }
    setElements() {
        this.elements.set('html', document.querySelector('html'));
        this.elements.set('outer-container', document.querySelector('.ig-consent-container'));
        this.elements.set('inner-container', document.querySelector('.ig-consent-container-inner'));
	this.elements.set('settings-container-list', document.querySelector('.settings-container-list'));

    }
    setButtons() {
	document.querySelector('.ig-consent-button-save')?.addEventListener('click', () => { this.save()});
	document.querySelector('.ig-consent-button-all')?.addEventListener('click', () => { this.all()});
	document.querySelector('.ig-consent-button-needed')?.addEventListener('click', () => { this.needed()});
	document.querySelector('.ig-consent-button-settings')?.addEventListener('click', () => {
	    this.settings();
	});
	document.querySelector('.ig-consent-button-close')?.addEventListener('click', () => {
	    if (this.manager.confirmed) {
		this.hide();
	    } else {
		this.close();
	    }
	});
    }
    updatePurposeBoxes() {
        // check if all checkboxes of a purpuse are checked
        let allChecked = [];
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.dataset.parent) {
                let purpose = checkbox.dataset.parent;
                if (typeof allChecked[purpose] === 'undefined' || allChecked[purpose]) {
                    allChecked[purpose] = checkbox.checked;
                }
            }
        });

        this.elements.get('checkboxes-purpose').forEach(checkbox => {
            let consent = checkbox.value;
            if (allChecked[consent]) {
                checkbox.checked = true;
            } else {
                checkbox.checked = false;
            }
        });
    }
    setServicesByPurpose(parent, value) {
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.dataset.parent == parent) {
                if (checkbox.checked != value) {
                    this.manager.updateConsent(checkbox.value, value);
                }
                checkbox.checked = value;
            }
        });
    }

    // update services from consent manager
    updateServiceBoxes() {
        this.elements.get('checkboxes-service').forEach(checkbox => {
            let consent = checkbox.value;
            if (this.manager.getConsent(consent)) {
                checkbox.checked = true;
            } else {
                checkbox.checked = false;
            }
        });

        //this.updatePurposeBoxes();
    }
    save() {
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.checked) {
                this.manager.updateConsent(checkbox.value, true);
            } else {
                this.manager.updateConsent(checkbox.value, false);
            }
        });
        this.manager.saveAndApplyConsents();
        this.hide();
    }
    close() {
        this.elements.get('inner-container').classList.remove('settings-show');
    }
    needed() {
        this.manager.saveAndApplyConsents();
        this.hide();
    }
    all() {
        this.manager.changeAll(true);
        this.manager.saveAndApplyConsents();
        this.hide();
    }
    settings() {
	let settingsContainer = this.elements.get('settings-container-list');
	let content = this.getTemplate();
	if (!settingsContainer.innerHTML) {
	    settingsContainer.innerHTML = content;
            this.elements.set('checkboxes-purpose', document.querySelectorAll('.ig-consent-container-inner .purpose input[type="checkbox"]:not(:disabled)'));
            this.elements.set('checkboxes-service', document.querySelectorAll('.ig-consent-container-inner .service input[type="checkbox"]:not(:disabled)'));
	// add checkbox click events
            this.elements.get('checkboxes-purpose').forEach((checkbox) => {
		checkbox.addEventListener('click', () => {
                    this.setServicesByPurpose(checkbox.value, checkbox.checked);
		});
            });
            this.elements.get('checkboxes-service').forEach((checkbox) => {
		checkbox.addEventListener('click', () => {
                    this.manager.updateConsent(checkbox.value, checkbox.checked);
                    this.updatePurposeBoxes();
                    //this.updateServiceBoxes();
		});
            });

	    
	}
        this.updateServiceBoxes();
	this.updatePurposeBoxes();
        this.elements.get('inner-container').classList.add('settings-show');
    }
}

const igConsentUi = new IgConsentUi();
if (typeof window.igJs !== 'undefined') {
    window.igJs.add('igConsentUi', igConsentUi);
}


// cookie settings link for menus
var cookieSettings = document.querySelector('[href="#cookie-settings"]');
cookieSettings?.addEventListener('click', (e) => {
    e.preventDefault();
    igConsentUi.showSettings();
});

